/******************************************************************************
*                         Steffen Hofmann                                     *
*                             shotech                                         *
*******************************************************************************
*  $Source: ,v $
*-----------------------------------------------------------------------------*
*  Datum der 1. Version  :  21.09.2017 21:09:41
*  Geraete Bezeichnung   :  UB88xx TB4 
*-----------------------------------------------------------------------------*
*- $Id:  ub88xx@shotech.de
*-----------------------------------------------------------------------------*
*
*	Sourcefile
*	----------
*	Grafiktreiber st7735s und ssd1283
* UB8820 oder UB8840
*
******************************************************************************/

#include "hw.h"
#include "tft_drv.h"
#include "font_6x8.h"


void ghw_io_init( void );
void ghw_cmd_wait(unsigned char ms);

// 16 rows (0 to 15) and 21 characters (0 to 20)
// Requires (11 + size*size*6*8) bytes of transmission for each character
u8 StX=0; // position along the horizonal axis 0 to 20
u8 StY=0; // position along the vertical axis 0 to 15
GCOLOR StTextColor;
GCOLOR gBgColor;

static uint8_t ColStart, RowStart; // some displays need this changed
static uint8_t Rotation;           // 0 to 3
//static enum initRFlags TabColor;
static u8 _width;   // this could probably be a constant, except it is used in Adafruit_GFX and depends on image rotation
static u8 _height;

TFT_TYP tTftTyp;


#define CMD 0
#define DAT 1

static const S1D_ST7735_REGS cmd_st7735[] =
   {
   {CMD,120,0x00},         /* Wait for any hardware startup completed */
   {CMD,100,GCTRL_ST7735_RESET},     /* 0x01 Reset */
	 {CMD,150,GCTRL_ST7735_SLPOUT},            //Sleep out
//	_delay_ms(120);

//	SendCommand(0x13); // Normal Display Mode On

	//------------------------------------ST7735S Frame Rate-----------------------------------------//
	 {CMD,0,0xB1},
	 {DAT,0,0x02},
	 {DAT,0,0x35},
	 {DAT,0,0x36},

	 {CMD,0,0xB2},
	 {DAT,0,0x02},
	 {DAT,0,0x35},
	 {DAT,0,0x36},

	 {CMD,0,0xB3},
	 {DAT,0,0x02},
	 {DAT,0,0x35},
	 {DAT,0,0x36},
	 {DAT,0,0x02},
	 {DAT,0,0x35},
	 {DAT,0,0x36},
//------------------------------------End ST7735S Frame Rate-----------------------------------------//
	 {CMD,0,0xB4}, //Column inversion
	 {DAT,0,0x07},

//------------------------------------ST7735S Power Sequence-----------------------------------------//
	 {CMD,0,0xC0},
	 {DAT,0,0xa2},
	 {DAT,0,0x02},
	 {DAT,0,0x84},

 	 {CMD,0,0xC1},
	 {DAT,0,0xC5},

	 {CMD,0,0xC2},
	 {DAT,0,0x0a},
	 {DAT,0,0x00},

	 {CMD,0,0xC3},
	 {DAT,0,0x8A},
	 {DAT,0,0x2A},

	 {CMD,0,0xC4},
	 {DAT,0,0x8A},
	 {DAT,0,0xEE},
//---------------------------------End ST7735S Power Sequence-------------------------------------//
	 {CMD,0,0xC5}, //VCOM
	 {DAT,0,0x03},

   {CMD,  0,GCTRL_ST7735_MADCTRL},  /* Set video buffer scan modes */
   {DAT,  0,  ST7735_MV_BIT | ST7735_RGB_BIT | ST7735_MX_BIT | ST7735_MY_BIT }, /* Rotation, x,y mirroring, rgb-bgr modes */
   
	 //{CMD,0,0x36}, //MX, MY, RGB mode
	 //{DAT,0xC8},	// Yes 1,5"

//------------------------------------ST7735S Gamma Sequence-----------------------------------------//
    {CMD,0,0xE0},
    {DAT,0,0x12},//p1
    {DAT,0,0x1C},//p2
    {DAT,0,0x10},//p3
    {DAT,0,0x18},//p4
    {DAT,0,0x33},//p5
    {DAT,0,0x2C},//p6
    {DAT,0,0x25},//p7
    {DAT,0,0x28},//p8
    {DAT,0,0x28},//p9
    {DAT,0,0x27},//p10
    {DAT,0,0x2F},//p11
    {DAT,0,0x3C},//p12
    {DAT,0,0x00},//p13
    {DAT,0,0x03},//p14
    {DAT,0,0x03},//p15
    {DAT,0,0x10},//p15

    {CMD,0,0xE1},
    {DAT,0,0x12},//p1
    {DAT,0,0x1c},//p2
    {DAT,0,0x10},//p3
    {DAT,0,0x18},//p4
    {DAT,0,0x2D},//p5
    {DAT,0,0x28},//p6
    {DAT,0,0x23},//p7
    {DAT,0,0x28},//p8
    {DAT,0,0x28},//p9
    {DAT,0,0x26},//p10
    {DAT,0,0x2F},//p11
    {DAT,0,0x3B},//p12
    {DAT,0,0x00},//p13
    {DAT,0,0x03},//p14
    {DAT,0,0x03},//p15
    {DAT,0,0x10},//p15   	

//------------------------------------End ST7735S Gamma Sequence-----------------------------------------//

//	SendCommand(0xFC); //Enable Gate Pump Clock Frequency Variable
//	SendData(0xC4);

	 {CMD,0,0xF0},	// Enable Test command
	 {DAT,0,0x10},

	 //{CMD,0,0x3A}, // Interface Pixel Format: 262k mode, 6-6-6 Bit
	 {CMD,  0,GCTRL_ST7735_COLMOD},  /*Pixel format */
	 {DAT,0,0x05}, //- 65k Mode RGB 5-6-5 Bit

	 {CMD,0,GCTRL_ST7735_DISPON}, //Display on

	 {CMD,0,GCTRL_ST7735_RAMWR}, 					   //Start GRAM write (79)
   };


static const S1D_SSD1283_REGS cmd_SSD1283[] =
   {
   
   {GCTRL_SSD1283_TEST_COMMAND   , 1, 0x0006},  // To enable 2xH Commands
   {GCTRL_SSD1283_TEST_COMMAND   , 1, 0x000E},  // Reset by Command
   {GCTRL_SSD1283_TEST_COMMAND   , 1, 0x0000},  // Release reset
   {GCTRL_SSD1283_PWR_CTRL0     ,100, 0x47B5},  // Enter Sleep Mode
   {GCTRL_SSD1283_PWR_CTRL0     ,100, 0x47B0},  // Leave Sleep Mode 
//   {GCTRL_SSD1283_PWR_CTRL0     ,100, 0x3AB5},  // Enter Sleep Mode
//   {GCTRL_SSD1283_PWR_CTRL0     ,100, 0x7AB0},  // Leave Sleep Mode
   {GCTRL_SSD1283_PWR_CTRL1      , 1, 0x001C},  // VGH/VCI ratio: X6
   {GCTRL_SSD1283_DISP_CTRL      , 1, 0x0021},  // Set Gate Output to VGOFFL
   {GCTRL_SSD1283_TEST_COMMAND   , 1, 0x0006},  // To enable 2xH Commands
   {GCTRL_SSD1283_TEST_COMMAND   , 1, 0x0005},  // 
   {GCTRL_SSD1283_START_OSC      ,10, 0x0001},
   {GCTRL_SSD1283_DISP_CTRL      , 1, 0x0023},
   {GCTRL_SSD1283_DISP_CTRL      , 1, 0x0033},   
   {GCTRL_SSD1283_VERTICAL_PORCH ,10, 0x0101},
   
   
   //{GCTRL_SSD1283_DRV_OUTPUT_CTRL, 1, 0x2B80}, 

   #if   ( defined( GHW_SSD1283_MIRROR_VER ) &&  defined( GHW_SSD1283_MIRROR_HOR ))
   {GCTRL_SSD1283_DRV_OUTPUT_CTRL, 0, 0x0380},
   #elif ( defined( GHW_SSD1283_MIRROR_VER ) && !defined( GHW_SSD1283_MIRROR_HOR ))
   {GCTRL_SSD1283_DRV_OUTPUT_CTRL, 0, 0x0280},
   #elif (!defined( GHW_SSD1283_MIRROR_VER ) &&  defined( GHW_SSD1283_MIRROR_HOR ))
   {GCTRL_SSD1283_DRV_OUTPUT_CTRL, 0, 0x0180},
   #else
   {GCTRL_SSD1283_DRV_OUTPUT_CTRL, 0, 0x2B80},
   #endif
   
   
   


   {GCTRL_SSD1283_VERTICAL_PORCH , 1, 0x0101},

   #ifdef GHW_SSD1283_ROTATED
   {GCTRL_SSD1283_ENTRYMODE      ,60, 0x6078}, /* 16 bit Color mode (vertical inc)*/
   #else
   {GCTRL_SSD1283_ENTRYMODE      ,60, 0x6070}, /* 16 bit Color mode (horizontal inc)*/
   #endif
   
   //{GCTRL_SSD1283_ENTRYMODE      , 1, 0x6030},   // 0110 0000 0011 0000

   
   {GCTRL_SSD1283_LCD_DRIVE_CTRL , 1, 0x0300},
   {GCTRL_SSD1283_FRAME_CTRL ,     1, 0x5828},
   {GCTRL_SSD1283_PWR_CTRL2      ,10, 0x060A}, //- 0x060A
   {GCTRL_SSD1283_PWR_CTRL6      ,10, 0x003C}, //- 0x003C
   {GCTRL_SSD1283_PWR_CTRL3      ,10, 0x3100},
   {GCTRL_SSD1283_PWR_CTRL5      ,10, 0x00BA},

   {GCTRL_SSD1283_GAMMA_0        , 1, 0x0404},
   {GCTRL_SSD1283_GAMMA_1        , 1, 0x0304},
   {GCTRL_SSD1283_GAMMA_2        , 1, 0x0001},
   {GCTRL_SSD1283_GAMMA_3        , 1, 0x0303},
   {GCTRL_SSD1283_GAMMA_4        , 1, 0x0002},
   {GCTRL_SSD1283_GAMMA_5        , 1, 0x0301},
   {GCTRL_SSD1283_GAMMA_6        , 1, 0x0707},
   {GCTRL_SSD1283_GAMMA_7        , 1, 0x0404},
   {GCTRL_SSD1283_GAMMA_8        , 1, 0x000F},
   {GCTRL_SSD1283_GAMMA_9        , 1, 0x0F00},
   
   {GCTRL_SSD1283_H_WIN_ADR      , 1, 0x8300},  // Horizontal RAM Address Position(0x8304:S4~G131)
   {GCTRL_SSD1283_V_WIN_ADR      , 1, 0x7F00},  // Vertical RAM Address Position(0x7f00:G0~G127)
   {GCTRL_SSD1283_RAM_ADR        , 1, 0x0000},
   
   {GCTRL_SSD1283_TEST_COMMAND   , 1, 0x0006},
   { 0x2E   , 50, 0x7A40},
//   {GCTRL_FRAME_FREQUENCY_CTRL   1, 0xB000},  // Internal Oszillator frequenz 236 Hz

//   {GCTRL_DISP_CTRL      , 0, 0x0014}, /* Disp drive on, scan halted, reverse rgb */
//   {GCTRL_DISP_CTRL      , 0, GDISP_OFF} /* Disp drive on,blank disp */
      
//   {GCTRL_RAM_ADR_L      , 0, 0x0000},
//   {GCTRL_RAM_ADR_H      , 0, 0x0000}
};
//-----------------------------------------------------------------------------
//- write Display Command
//-----------------------------------------------------------------------------
void ghw_writeCommand( u8 bData )
{
	
			//TFT_DC_COM();
//#pragma asm " \n \
//        nop    ;   \n \
//				nop	   ;   \n \
//				nop    ;   \n \
//				nop    ;   \n \
//"			
			DISPLAY_COM = bData;

}
//-----------------------------------------------------------------------------
//- write Display Data
//-----------------------------------------------------------------------------
void ghw_writeData( u8 bData )
{

	  		//TFT_DC_DAT();
//#pragma asm " \n \
//        nop    ;   \n \
//				nop	   ;   \n \
//				nop    ;   \n \
//				nop    ;   \n \
//"			
	  		DISPLAY_DAT = bData;
}
//-----------------------------------------------------------------------------
//-
//-----------------------------------------------------------------------------
void ghw_io_init( void )
{
	 P2 &= ~PORT_P2_TFT_RST;
	 ghw_cmd_wait(50);
	 P2 |= PORT_P2_TFT_RST;
}


void ghw_cmd_wait(unsigned char ms)
{

 unsigned short wait1ms;
   while (ms-- != 0)
      {
      wait1ms = 40;     /* Adjust to achieve a 1 ms loop below */
      while( wait1ms != 0)
         wait1ms--;
      }

}
   
   
void st7735_ghw_init(void)
{
   u16 i;
   tTftTyp = TFT_ST7735;

   StX=0; // position along the horizonal axis 0 to 20
   StY=0; // position along the vertical axis 0 to 15
   StTextColor = G_YELLOW;
   gBgColor = G_BLACK;

   ColStart=0; 
   RowStart=32; // some displays need this changed
   Rotation=0;           // 0 to 3
   
   _width = 128;   // this could probably be a constant, except it is used in Adafruit_GFX and depends on image rotation
   _height = 128;
   
   ghw_io_init(); /* Set any hardware interface lines, controller hardware reset */
   /* Set default colors */
   //st7735_ghw_setcolor( GHW_PALETTE_FOREGROUND, GHW_PALETTE_BACKGROUND );   
   //st7735_ghw_palette_wr(0, sizeof(ghw_palette)/sizeof(GPALETTE_RGB), (GCONSTP GPALETTE_RGB PFCODE *)&ghw_palette[0]);
   
   for (i=0; i < (sizeof(cmd_st7735)) / (sizeof(S1D_ST7735_REGS)); i++)
      {
      if (cmd_st7735[i].index == CMD)
         //ghw_cmd(cmd_st7735[i].value);
         //DISPLAY_COM = cmd_st7735[i].value;
         ghw_writeCommand( cmd_st7735[i].value );
      else
         //ghw_cmddat(cmd_st7735[i].value);
         //DISPLAY_DAT = cmd_st7735[i].value;
         ghw_writeData( cmd_st7735[i].value );
      if (cmd_st7735[i].delay != 0)
         ghw_cmd_wait( cmd_st7735[i].delay );
      }
}

void ssd1283_ghw_init(void)
{
   short i;
   
   ghw_io_init(); /* Set any hardware interface lines, controller hardware reset */
   
   /* Initialize controller according to configuration file */
   for (i=0; i < sizeof(cmd_SSD1283)/sizeof(S1D_SSD1283_REGS); i++)
      {
//      ghw_cmd_dat_wr(cmd_SSD1283[i].index,cmd_SSD1283[i].value);
//      if (cmd_SSD1283[i].delay != 0)
//         ghw_cmd_wait( cmd_SSD1283[i].delay );
//        DISPLAY_COM = 0x00;                 /* Msb */
//        DISPLAY_COM = cmd_SSD1283[i].index; /* Lsb */
//        DISPLAY_DAT = (unsigned char)( cmd_SSD1283[i].value >> 8); /* Msb */
//        DISPLAY_DAT = (unsigned char)( cmd_SSD1283[i].value);      /* Lsb */
        ghw_writeCommand( 0 );
        ghw_writeCommand( cmd_SSD1283[i].index );
        ghw_writeData( (unsigned char)( cmd_SSD1283[i].value >> 8) );
        ghw_writeData( (unsigned char)( cmd_SSD1283[i].value) );
      if (cmd_SSD1283[i].delay != 0)
           ghw_cmd_wait( cmd_SSD1283[i].delay );
      }   
}

// Set the region of the screen RAM to be modified
// Pixel colors are sent left to right, top to bottom
// (same as Font table is encoded; different from regular bitmap)
// Requires 11 bytes of transmission
void static setAddrWindow(uint8_t x0, uint8_t y0, uint8_t x1, uint8_t y1) {
  
  if( tTftTyp == TFT_ST7735 )
  	{
		  ghw_writeCommand( GCTRL_ST7735_CASET );
		  ghw_writeData( 0 );
		  ghw_writeData( x0+ColStart );
		  ghw_writeData( 0 );
		  ghw_writeData( x1+ColStart );

		  ghw_writeCommand( GCTRL_ST7735_RASET );
		  ghw_writeData( 0 );
		  ghw_writeData( y0+RowStart );
		  ghw_writeData( 0 );
		  ghw_writeData( y1+RowStart );
      
      ghw_writeCommand( GCTRL_ST7735_RAMWR );

		}
		else
			{
//        set_xyrange(GXT xb, GYT yb, GXT xe, GYT ye)
//        ghw_cmd_dat_wr(GCTRL_V_WIN_ADR,(((SGUINT)ye+GHW_YOFFSET) << 8) | (yb+GHW_YOFFSET));
//        /* Set address pointer to start of range */
//         ghw_cmd_dat_wr(GCTRL_RAM_ADR, (((SGUINT)yb+GHW_YOFFSET) << 8) |  (xb+GHW_XOFFSET));
//				 
//				 ghw_cmd_wr(GCTRL_SSD1283_RAM_ADR);
				ghw_writeCommand( 0 );
				ghw_writeCommand( GCTRL_SSD1283_V_WIN_ADR );
				ghw_writeData( (unsigned char)( ((((u16)y1+GHW_SSD1283_YOFFSET) << 8) | (y0+GHW_SSD1283_YOFFSET)) >> 8) );
				ghw_writeData( (unsigned char)( ((((u16)y1+GHW_SSD1283_YOFFSET) << 8) | (y0+GHW_SSD1283_YOFFSET))) );
				 
//        DISPLAY_COM = 0x00;                      /* Msb */
//        DISPLAY_COM = GCTRL_SSD1283_V_WIN_ADR;   /* Lsb */
//        DISPLAY_DAT = (unsigned char)( ((((u16)y1+GHW_SSD1283_YOFFSET) << 8) | (y0+GHW_SSD1283_YOFFSET)) >> 8);  /* Msb */
//        DISPLAY_DAT = (unsigned char)( ((((u16)y1+GHW_SSD1283_YOFFSET) << 8) | (y0+GHW_SSD1283_YOFFSET)));

				ghw_writeCommand( 0 );
				ghw_writeCommand( GCTRL_SSD1283_RAM_ADR );
				ghw_writeData( (unsigned char)( ((((u16)y0+GHW_SSD1283_YOFFSET) << 8) |  (x0+GHW_SSD1283_XOFFSET)) >> 8 ) );
				ghw_writeData( (unsigned char)( ((((u16)y0+GHW_SSD1283_YOFFSET) << 8) |  (x0+GHW_SSD1283_XOFFSET))) );
				
//        DISPLAY_COM = 0x00;                      /* Msb */
//        DISPLAY_COM = GCTRL_SSD1283_RAM_ADR;   /* Lsb */        
//        DISPLAY_DAT = (unsigned char)( ((((u16)y0+GHW_SSD1283_YOFFSET) << 8) |  (x0+GHW_SSD1283_XOFFSET)) >> 8 );      /* Lsb */
//        DISPLAY_DAT = (unsigned char)( ((((u16)y0+GHW_SSD1283_YOFFSET) << 8) |  (x0+GHW_SSD1283_XOFFSET)));
        
        ghw_writeCommand( 0 );
        ghw_writeCommand( GCTRL_SSD1283_RAMWR );
        
//        DISPLAY_COM = 0x00; 
//        DISPLAY_COM = GCTRL_SSD1283_RAMWR; 
			}
}

// Send two bytes of data, most significant byte first
// Requires 2 bytes of transmission
void static pushColor(GCOLOR color) {

  ghw_writeData(((uint8_t)(color >> 8)) );
  ghw_writeData(((uint8_t)color) );

}

//------------ST7735_DrawPixel------------
// Color the pixel at the given coordinates with the given color.
// Requires 13 bytes of transmission
// Input: x     horizontal position of the pixel, columns from the left edge
//               must be less than 128
//               0 is on the left, 126 is near the right
//        y     vertical position of the pixel, rows from the top edge
//               must be less than 160
//               159 is near the wires, 0 is the side opposite the wires
//        color 16-bit color, which can be produced by ST7735_Color565()
// Output: none
void DrawPixel(u8 x, u8 y, GCOLOR color) {

  if((x < 0) || (x >= _width) || (y < 0) || (y >= _height)) return;

//  setAddrWindow(x,y,x+1,y+1); // original code, bug???
  setAddrWindow(x,y,x,y);

  pushColor(color);
}

//------------ST7735_DrawFastVLine------------
// Draw a vertical line at the given coordinates with the given height and color.
// A vertical line is parallel to the longer side of the rectangular display
// Requires (11 + 2*h) bytes of transmission (assuming image fully on screen)
// Input: x     horizontal position of the start of the line, columns from the left edge
//        y     vertical position of the start of the line, rows from the top edge
//        h     vertical height of the line
//        color 16-bit color, which can be produced by ST7735_Color565()
// Output: none
void DrawFastVLine(u8 x, u8 y, u8 h, GCOLOR color) {
  uint8_t hi = color >> 8, lo = color;

  // Rudimentary clipping
  if((x >= _width) || (y >= _height)) return;
  if((y+h-1) >= _height) h = _height-y;
  setAddrWindow(x, y, x, y+h-1);

  while (h--) {
  ghw_writeData(hi );
  ghw_writeData(lo );

//    DISPLAY_DAT = hi;
//    DISPLAY_DAT = lo;
  }
}

//------------ST7735_DrawFastHLine------------
// Draw a horizontal line at the given coordinates with the given width and color.
// A horizontal line is parallel to the shorter side of the rectangular display
// Requires (11 + 2*w) bytes of transmission (assuming image fully on screen)
// Input: x     horizontal position of the start of the line, columns from the left edge
//        y     vertical position of the start of the line, rows from the top edge
//        w     horizontal width of the line
//        color 16-bit color, which can be produced by ST7735_Color565()
// Output: none
void DrawFastHLine(u8 x, u8 y, u8 w, GCOLOR color) {
  uint8_t hi = color >> 8, lo = color;

  // Rudimentary clipping
  if((x >= _width) || (y >= _height)) return;
  if((x+w-1) >= _width)  w = _width-x;
  setAddrWindow(x, y, x+w-1, y);

  while (w--) {
  ghw_writeData(hi );
  ghw_writeData(lo );

  }
}


//------------ST7735_FillScreen------------
// Fill the screen with the given color.
// Requires 40,971 bytes of transmission
// Input: color 16-bit color, which can be produced by ST7735_Color565()
// Output: none
void FillScreen(GCOLOR color) {
  FillRect(0, 0, _width, _height, color);  // original
//  screen is actually 129 by 161 pixels, x 0 to 128, y goes from 0 to 160
}

//------------ST7735_FillRect------------
// Draw a filled rectangle at the given coordinates with the given width, height, and color.
// Requires (11 + 2*w*h) bytes of transmission (assuming image fully on screen)
// Input: x     horizontal position of the top left corner of the rectangle, columns from the left edge
//        y     vertical position of the top left corner of the rectangle, rows from the top edge
//        w     horizontal width of the rectangle
//        h     vertical height of the rectangle
//        color 16-bit color, which can be produced by ST7735_Color565()
// Output: none
void FillRect(u8 x, u8 y, u8 w, u8 h, GCOLOR color) {
  uint8_t hi = color >> 8, lo = color;

  
  // rudimentary clipping (drawChar w/big text requires this)
  if((x >= _width) || (y >= _height)) return;
  if((x + w - 1) >= _width)  w = _width  - x;
  if((y + h - 1) >= _height) h = _height - y;

  setAddrWindow(x, y, x+w-1, y+h-1);

  for(y=h; y>0; y--) {
    for(x=w; x>0; x--) {
  ghw_writeData(hi );
  ghw_writeData(lo );

    }
  }
}

//------------ST7735_Color565------------
// Pass 8-bit (each) R,G,B and get back 16-bit packed color.
// Input: r red value
//        g green value
//        b blue value
// Output: 16-bit color
uint16_t Color565(uint8_t r, uint8_t g, uint8_t b) {
  return ((b & 0xF8) << 8) | ((g & 0xFC) << 3) | (r >> 3);
}


//------------ST7735_SwapColor------------
// Swaps the red and blue values of the given 16-bit packed color;
// green is unchanged.
// Input: x 16-bit color in format B, G, R
// Output: 16-bit color in format R, G, B
uint16_t SwapColor(uint16_t x) {
  return (x << 11) | (x & 0x07E0) | (x >> 11);
}

//------------ST7735_DrawCharS------------
// Simple character draw function.  This is the same function from
// Adafruit_GFX.c but adapted for this processor.  However, each call
// to ST7735_DrawPixel() calls setAddrWindow(), which needs to send
// many extra data and commands.  If the background color is the same
// as the text color, no background will be printed, and text can be
// drawn right over existing images without covering them with a box.
// Requires (11 + 2*size*size)*6*8 (image fully on screen; textcolor != bgColor)
// Input: x         horizontal position of the top left corner of the character, columns from the left edge
//        y         vertical position of the top left corner of the character, rows from the top edge
//        c         character to be printed
//        textColor 16-bit color of the character
//        bgColor   16-bit color of the background
//        size      number of pixels per character pixel (e.g. size==2 prints each pixel of font as 2x2 square)
// Output: none
void DrawCharS(u8 x, u8 y, char c, GCOLOR textColor, GCOLOR bgColor, uint8_t size){
  uint8_t line; // vertical column of pixels of character in font
  int32_t i, j;
  if((x >= _width)            || // Clip right
     (y >= _height)           || // Clip bottom
     ((x + 5 * size - 1) < 0) || // Clip left
     ((y + 8 * size - 1) < 0))   // Clip top
    return;

  for (i=0; i<6; i++ ) {
    if (i == 5)
      line = 0x0;
    else
      line = Font_6x8[(c*5)+i];
    for (j = 0; j<8; j++) {
      if (line & 0x1) {
        if (size == 1) // default size
          DrawPixel(x+i, y+j, textColor);
        else {  // big size
          FillRect(x+(i*size), y+(j*size), size, size, textColor);
        }
      } else if (bgColor != textColor) {
        if (size == 1) // default size
          DrawPixel(x+i, y+j, bgColor);
        else {  // big size
          FillRect(x+i*size, y+j*size, size, size, bgColor);
        }
      }
      line >>= 1;
    }
  }
}

//------------ST7735_DrawChar------------
// Advanced character draw function.  This is similar to the function
// from Adafruit_GFX.c but adapted for this processor.  However, this
// function only uses one call to setAddrWindow(), which allows it to
// run at least twice as fast.
// Requires (11 + size*size*6*8) bytes of transmission (assuming image fully on screen)
// Input: x         horizontal position of the top left corner of the character, columns from the left edge
//        y         vertical position of the top left corner of the character, rows from the top edge
//        c         character to be printed
//        textColor 16-bit color of the character
//        bgColor   16-bit color of the background
//        size      number of pixels per character pixel (e.g. size==2 prints each pixel of font as 2x2 square)
// Output: none
void DrawChar(u8 x, u8 y, char c, GCOLOR textColor, GCOLOR bgColor, uint8_t size){
  uint8_t line; // horizontal row of pixels of character
  u16 i, j;// loop indices
  u8 col, row;
  if(((x + 5*size - 1) >= _width)  || // Clip right
     ((y + 8*size - 1) >= _height) || // Clip bottom
     ((x + 5*size - 1) < 0)        || // Clip left
     ((y + 8*size - 1) < 0)){         // Clip top
    return;
  }

  setAddrWindow(x, y, x+6*size-1, y+8*size-1);

  line = 0x01;        // print the top row first
  // print the rows, starting at the top
  for(row=0; row<8; row=row+1){
    for(i=0; i<size; i=i+1){
      // print the columns, starting on the left
      for(col=0; col<5; col=col+1){
        if(Font_6x8[(c*5)+col]&line){
          // bit is set in Font, print pixel(s) in text color
          for(j=0; j<size; j=j+1){
            pushColor(textColor);
          }
        } else{
          // bit is cleared in Font, print pixel(s) in background color
          for(j=0; j<size; j=j+1){
            pushColor(bgColor);
          }
        }
      }
      // print blank column(s) to the right of character
      for(j=0; j<size; j=j+1){
        pushColor(bgColor);
      }
    }
    line = line<<1;   // move up to the next row
  }
}

//------------ST7735_DrawString------------
// String draw function.
// 16 rows (0 to 15) and 21 characters (0 to 20)
// Requires (11 + size*size*6*8) bytes of transmission for each character
// Input: x         columns from the left edge (0 to 20)
//        y         rows from the top edge (0 to 15)
//        pt        pointer to a null terminated string to be printed
//        textColor 16-bit color of the characters
// bgColor is Black and size is 1
// Output: number of characters printed
u8 DrawString(u8 x, u8 y, char *pt, GCOLOR textColor, GCOLOR bgColor){
  u8 count = 0;
  if(y>15) return 0;
  while(*pt){
    DrawCharS(x*6, y*10, *pt, textColor, bgColor, 1);
    pt++;
    x = x+1;
    if(x>20) return count;  // number of characters printed
    count++;
  }
  return count;  // number of characters printed
}

// *************** ST7735_OutChar ********************
// Output one character to the LCD
// Position determined by ST7735_SetCursor command
// Color set by ST7735_SetTextColor
// Inputs: 8-bit ASCII character
// Outputs: none
void OutChar(char ch){
  if((ch == 10) || (ch == 13) || (ch == 27)){
    StY++; StX=0;
    if(StY>15){
      StY = 0;
    }
    DrawString(0,StY,"                     ",StTextColor, gBgColor);
    return;
  }
  DrawCharS(StX*6,StY*10,ch,G_YELLOW,G_BLACK, 1);
  StX++;
  if(StX>20){
    StX = 20;
    DrawCharS(StX*6,StY*10,'*',G_RED,G_BLACK, 1);
  }
  return;
}

//********ST7735_OutString*****************
// Print a string of characters to the ST7735 LCD.
// Position determined by ST7735_SetCursor command
// Color set by ST7735_SetTextColor
// The string will not automatically wrap.
// inputs: ptr  pointer to NULL-terminated ASCII string
// outputs: none
void OutString(char *ptr){
  while(*ptr){
    OutChar(*ptr);
    ptr = ptr + 1;
  }
}
// ************** ST7735_SetTextColor ************************
// Sets the color in which the characters will be printed
// Background color is fixed at black
// Input:  16-bit packed color
// Output: none
// ********************************************************
void SetTextColor(GCOLOR color){
  StTextColor = color;
}

void SetBgColor(GCOLOR color){
  gBgColor = color;
}


// *************** Output_Init ********************
// Standard device driver initialization function for printf
// Initialize ST7735 LCD
// Inputs: none
// Outputs: none
void Output_Init(void){
//  ST7735_InitR(INITR_REDTAB);
//  ST7735_FillScreen(0);                 // set screen to black
  //- Displayerkennung
  st7735_ghw_init();
  ssd1283_ghw_init();
  
  FillScreen(0);
}

// Clear display
void Output_Clear(void){ // Clears the display
  FillScreen(0);    // set screen to black
}

// set the color for future output
// Background color is fixed at black
// Input:  16-bit packed color
// Output: none
void Output_Color(GCOLOR newColor){ // Set color of future output
  SetTextColor(newColor);
}
