/******************************************************************************
*                         Steffen Hofmann                                     *
*                             shotech                                         *
*******************************************************************************
*  $Source: ,v $
*-----------------------------------------------------------------------------*
*  Datum der 1. Version  :  21.09.2017 21:09:41
*  Geraete Bezeichnung   :  UB88xx TB4 
*-----------------------------------------------------------------------------*
*- $Id:  ub88xx@shotech.de
*-----------------------------------------------------------------------------*
*
*	Headerfile
*	----------
*	Grafiktreiber st7735s und ssd1283
* UB8820 oder UB8840
*
******************************************************************************/
#ifndef TFT_DRV_H
#define TFT_DRV_H

#include "types.h"

#define GDISPPIXW 16

//   #define G_BLACK       0x0000
//   #define G_RED         0xf800
//   #define G_GREEN       0x07e0
//   #define G_YELLOW      0xffe0
//   #define G_BLUE        0x001f
//   #define G_MAGENTA     0xf81f
//   #define G_CYAN        0x07ff
//   #define G_WHITE       0xffff
//   /* Names for extended colors */
//   #define G_LLIGHTGREY  0xd6ba
//   #define G_LIGHTGREY   0xbdf7
//   #define G_GREY        0xad75
//   #define G_DARKGREY    0x94b2
//   #define G_DDARKGREY   0x7bef
//   #define G_DARKBLUE    0x0018
//   #define G_ORANGE      0xfc00
   
   
#define GHW_PALETTE_BACKGROUND  G_WHITE
#define GHW_PALETTE_FOREGROUND  G_BLACK
// some flags for ST7735_InitR()
enum initRFlags{
  none,
  INITR_GREENTAB,
  INITR_REDTAB,
  INITR_BLACKTAB
};

typedef enum {
	TFT_NO,
	TFT_ST7735,
	TFT_SSD1283,
	
	TFT_MAX
}TFT_TYP;

typedef struct
   {
   unsigned char  index;
   unsigned char  delay;
   unsigned short value;
   } S1D_SSD1283_REGS;

typedef struct
   {
   unsigned char  index;
   unsigned char  delay;
   unsigned char  value;
   } S1D_ST7735_REGS;   

/* Common registers default S6D0129, S6D0118, R61506 SSD1283*/
#define GHW_SSD1283_MIRROR_VER   /* Mirror the display vertically */
/* #define GHW_MIRROR_HOR */  /* Mirror the display horizontally */
#define GHW_SSD1283_XOFFSET  4      /* Set display x start offset in on-chip video ram */
#define GHW_SSD1283_YOFFSET  0      /* Set display y start offset in on-chip video ram */
/*  #define GHW_SSD1283_ROTATED */    /* Define to rotate display 90 (270) degrees (remember to swap values used in GDISPH,GDISPW definitions) */
#define GHW_SSD1283_COLOR_SWAP      /* Define to change R,G,B order to B,G,R order */
  
#define  GCTRL_SSD1283_START_OSC             0x00  /* wr only */
#define  GCTRL_SSD1283_DEVICE_CODE           0x00  /* rd only */
#define  GCTRL_SSD1283_DRV_OUTPUT_CTRL       0x01  /* Driver output control */
#define  GCTRL_SSD1283_LCD_DRIVE_CTRL        0x02  /*  */

#define  GCTRL_SSD1283_ENTRYMODE             0x03  /* command */
#define  GCTRL_SSD1283_ENTRY_DFM1            0x4000  //- Bit DFM1
#define  GCTRL_SSD1283_ENTRY_DFM0            0x2000  //- Bit DFM0
#define  GCTRL_SSD1283_ENTRY_TY1             0x0080  //- Bit TY1
#define  GCTRL_SSD1283_ENTRY_TY0             0x0040  //- Bit TY0
#define  GCTRL_SSD1283_ENTRY_LG2             0x0004  //- Bit LG2
#define  GCTRL_SSD1283_ENTRY_LG1             0x0002  //- Bit LG1
#define  GCTRL_SSD1283_ENTRY_LG0             0x0001  //- Bit LG0
#define  GCTRL_SSD1283_ENTRY_ID1             0x0020  //- Bit ID1
#define  GCTRL_SSD1283_ENTRY_ID0             0x0010  //- Bit ID0
#define  GCTRL_SSD1283_ENTRY_AM              0x0008  //- Bit AM

#define  GCTRL_SSD1283_ENTRY_65K_Color       (GCTRL_SSD1283_ENTRY_DFM1 | GCTRL_SSD1283_ENTRY_DFM1)
#define  GCTRL_SSD1283_ENTRY_262K_Color      GCTRL_SSD1283_ENTRY_DFM1
//- Gilt nur im 262k Mode 16Bit Bus
#define  GCTRL_SSD1283_ENTRY_262k_CM_A       0
#define  GCTRL_SSD1283_ENTRY_262k_CM_B       GCTRL_SSD1283_ENTRY_TY0
#define  GCTRL_SSD1283_ENTRY_262k_CM_C       ( GCTRL_SSD1283_ENTRY_TY0 | GCTRL_SSD1283_ENTRY_TY1 )


#define  GCTRL_SSD1283_DISP_CTRL             0x07  /*  */
#define  GCTRL_SSD1283_BLANK_CTRL            0x08  /*  */
#define  GCTRL_SSD1283_GATE_NON_DISP         0x09  /*  */
#define  GCTRL_SSD1283_FRAME_CTRL            0x0b  /*  */
#define  GCTRL_SSD1283_DRV_OUT_CTRL          0x0d  /* S60129 only */
#define  GCTRL_SSD1283_EXT_INTF              0x0c  /* R61506, TL1771 only */
#define  GCTRL_SSD1283_PWR_CTRL0             0x10  /*  */
#define  GCTRL_SSD1283_PWR_CTRL1             0x11  /*  */
#define  GCTRL_SSD1283_PWR_CTRL2             0x12  /* R61506, TL1771 only  */
#define  GCTRL_SSD1283_PWR_CTRL3             0x13  /*  */
#define  GCTRL_SSD1283_PWR_CTRL4             0x14  /* S6D0129, TL1771 only */

#define  GCTRL_SSD1283_RAMWR                 0x22  /* Set or get GRAM data */
#define  GCTRL_SSD1283_RAMRD                 0x22  /* Set or get GRAM data */

#define  GCTRL_SSD1283_GAMMA_0               0x30  /* Set gamma ctrl */
#define  GCTRL_SSD1283_GAMMA_1               0x31  /* Set gamma ctrl */
#define  GCTRL_SSD1283_GAMMA_2               0x32  /* Set gamma ctrl */
#define  GCTRL_SSD1283_GAMMA_3               0x33  /* Set gamma ctrl */
#define  GCTRL_SSD1283_GAMMA_4               0x34  /* Set gamma ctrl */
#define  GCTRL_SSD1283_GAMMA_5               0x35  /* Set gamma ctrl */
#define  GCTRL_SSD1283_GAMMA_6               0x36  /* Set gamma ctrl */
#define  GCTRL_SSD1283_GAMMA_7               0x37  /* Set gamma ctrl */
#define  GCTRL_SSD1283_GAMMA_8               0x38  /* Set gamma ctrl */
#define  GCTRL_SSD1283_GAMMA_9               0x39  /* Set gamma ctrl */

#define  GCTRL_SSD1283_GATE_SCAN             0x40  /*  */
#define  GCTRL_SSD1283_VSCROLL               0x41  /*  */

#define  GCTRL_SSD1283_PWR_CTRL5             0x1E  /*  */
#define  GCTRL_SSD1283_PWR_CTRL6             0x1F
#define  GCTRL_SSD1283_TEST_COMMAND          0x28
#define  GCTRL_SSD1283_FRAME_FREQUENCY_CTRL  0x2C
#define  GCTRL_SSD1283_VERTICAL_PORCH        0x17

#define  GCTRL_SSD1283_H_WIN_ADR      0x44  /* end, begin */
#define  GCTRL_SSD1283_V_WIN_ADR      0x45  /* end, begin  */
#define  GCTRL_SSD1283_RAM_ADR        0x21  /* Set GRAM address */

   /* Display control registers for control of primary drawing operations ST7735*/
   /* Mirror and rotation definition bits (to simplify initialization below) */
#define GHW_ST7735_MIRROR_VER    /* Mirror the display vertically */
#define GHW_ST7735_MIRROR_HOR    /* Mirror the display horizontally */
#define GHW_ST7735_XOFFSET  32    /* Set display x start offset in on-chip video ram */
#define GHW_ST7735_YOFFSET  0    /* Set display y start offset in on-chip video ram */
/*#define GHW_ROTATED */   /* Define to rotate display 90 (270) degrees (remember to swap values used in GDISPH,GDISPW definitions) */
#define GHW_ST7735_COLOR_SWAP    /* Define to change R,G,B order to B,G,R order */
 
#ifdef GHW_ST7735_COLOR_SWAP
   #define  ST7735_RGB_BIT 0x08
#else
   #define  ST7735_RGB_BIT 0x00
#endif
#ifdef GHW_ST7735_MIRROR_HOR
   #define  ST7735_MX_BIT 0x40
#else
   #define  ST7735_MX_BIT 0x00
#endif
#ifdef GHW_ST7735_MIRROR_VER
   #define  ST7735_MY_BIT 0x80
#else
   #define  ST7735_MY_BIT 0x00
#endif
#ifdef GHW_ST7735_ROTATED
   #define  ST7735_MV_BIT 0x20
#else
   #define  ST7735_MV_BIT 0x00
#endif

#define GCTRL_ST7735_DISPOFF   0x28
#define GCTRL_ST7735_DISPON    0x29
#define GCTRL_ST7735_CASET     0x2A
#define GCTRL_ST7735_RASET     0x2B
#define GCTRL_ST7735_RAMWR     0x2C
#define GCTRL_ST7735_RAMRD     0x2E

   /* Display control registers for initialization only */
#define GCTRL_ST7735_RESET     0x01
#define GCTRL_ST7735_SLPIN     0x10
#define GCTRL_ST7735_SLPOUT    0x11
#define GCTRL_ST7735_NORON     0x13
#define GCTRL_ST7735_INVON     0x21
#define GCTRL_ST7735_GAMSET    0x26
#define GCTRL_ST7735_COLORSET  0x2D
#define GCTRL_ST7735_MADCTRL   0x36
#define GCTRL_ST7735_COLMOD    0x3A

#define GCTRL_ST7735_SETOSC    0xB0
#define GCTRL_ST7735_SETPOWER  0xB1
#define GCTRL_ST7735_SETDISP   0xB2
#define GCTRL_ST7735_SETCYC    0xB4
#define GCTRL_ST7735_SETVCOM   0xB6


#if (GDISPPIXW == 16)

   #define G_BLACK       0x0000
   #define G_RED         0xf800
   #define G_GREEN       0x07e0
   #define G_YELLOW      0xffe0
   #define G_BLUE        0x001f
   #define G_MAGENTA     0xf81f
   #define G_CYAN        0x07ff
   #define G_WHITE       0xffff
   /* Names for extended colors */
   #define G_LLIGHTGREY  0xd6ba
   #define G_LIGHTGREY   0xbdf7
   #define G_GREY        0xad75
   #define G_DARKGREY    0x94b2
   #define G_DDARKGREY   0x7bef
   #define G_DARKBLUE    0x0018
   #define G_ORANGE      0xfc00
typedef u16 GCOLOR;

#elif (GDISPPIXW == 18)

   #define G_BLACK       0x00000
   #define G_RED         0x3f000
   #define G_GREEN       0x00fc0
   #define G_YELLOW      0x3ffc0
   #define G_BLUE        0x0003f
   #define G_MAGENTA     0x3f03f
   #define G_CYAN        0x00fff
   #define G_WHITE       0x3ffff
   /* Names for extended colors */
   #define G_LLIGHTGREY  0x34d34
   #define G_LIGHTGREY   0x2ebae
   #define G_GREY        0x2aaaa
   #define G_DARKGREY    0x24924
   #define G_DDARKGREY   0x1e79e
   #define G_DARKBLUE    0x00030
   #define G_ORANGE      0x3f800

typedef u32 GCOLOR;

#elif (GDISPPIXW == 24)

   #define G_BLACK       0x000000
   #define G_RED         0xff0000
   #define G_GREEN       0x00ff00
   #define G_YELLOW      0xffff00
   #define G_BLUE        0x0000ff
   #define G_MAGENTA     0xff00ff
   #define G_CYAN        0x00ffff
   #define G_WHITE       0xffffff
   /* Names for extended colors */
   #define G_LLIGHTGREY  0xd0d0d0
   #define G_LIGHTGREY   0xb8b8b8
   #define G_GREY        0xa8a8a8
   #define G_DARKGREY    0x909090
   #define G_DDARKGREY   0x787878
   #define G_DARKBLUE    0x0000c0
   #define G_ORANGE      0xff8000

typedef u16 GCOLOR;

#endif


void ghw_writeCommand( u8 bData );
void ghw_writeData( u8 bData );

void st7735_ghw_init(void);
void ssd1283_ghw_init(void);

void FillRect(u8 x, u8 y, u8 w, u8 h, GCOLOR color);
void FillScreen(GCOLOR color);

u8 DrawString(u8 x, u8 y, char *pt, GCOLOR textColor, GCOLOR bgColor);
void Output_Clear(void);

void SetBgColor(GCOLOR color);

#endif
