/******************************************************************************
*                         Steffen Hofmann                                     *
*                             shotech                                         *
*******************************************************************************
*  $Source: ,v $
*-----------------------------------------------------------------------------*
*  Datum der 1. Version  :  17.09.2017 13:53:08
*  Geraete Bezeichnung   :  UB88xx TB4 
*-----------------------------------------------------------------------------*
*- $Id:  ub88xx@shotech.de
*-----------------------------------------------------------------------------*
*
*	Headerfile
*	----------
*	Fr Hardware Festlegungen des Projekts
* UB8820 oder UB8840
*
******************************************************************************/
#ifndef HW_H
#define HW_H

#include "ub88xx.h"
#include "types.h"

//- Belegung der Prozessorports Port 0 ----------------------------------------
//         Port-Name      Funktion       Init                      Comments
// P00     DB0            data/adr       io                        Daten/ADR 0..7
// P01     DB1            data/adr       io                        Daten/ADR 0..7
// P02     DB2            data/adr       io                        Daten/ADR 0..7
// P03     DB3            data/adr       io                        Daten/ADR 0..7
// P04     DB4            data/adr       io                        Daten/ADR 0..7
// P05     DB5            data/adr       io                        Daten/ADR 0..7
// P06     DB6            data/adr       io                        Daten/ADR 0..7
// P07     DB7            data/adr       io                        Daten/ADR 0..7

//- Belegung der Prozessorports Port 1 ----------------------------------------
//         Port-Name      Funktion       Init                      Commentss
// P10     AB8            adr            output                    adr 8..15
// P11     AB9            adr            output                    adr 8..15
// P12     AB10           adr            output                    adr 8..15
// P13     AB11           adr            output                    adr 8..15
// P14     AB12           adr            output                    adr 8..15
// P15     AB13           adr            output                    adr 8..15
// P16     AB14           adr            output                    adr 8..15
// P17     AB15           adr            output                    adr 8..15

//- Belegung der Prozessorports Port 2 ----------------------------------------
//         Port-Name      Funktion       Init                      Comments
//-----------------------------------------------------------------------------
// P20     RST_TFT        port           output                    tft   
// P21     SPI_CS         port           output                    spi   
// P22     MISO           port           input                     spi 
// P23     SDA            port           io                        I2C optional
// P24     SCL            port           io                        I2C optional
// P25     frei           port           output                    frei 
// P26     frei           port           output                    frei
// P27     PS2_DATA       port           io                        PS2 optional
//- Belegung der Prozessorports Port 3 ----------------------------------------
//         Port-Name      Funktion       Init                      Comments
//-----------------------------------------------------------------------------
// P30     RXD            port           input                     RS232/USB
// P31     frei           port           input                     -frei-
// P32     PS2_CLOCK      port           input                     PS2 optional
// P33     EOC            port           input                     AD-Wandler End of Conv. 
// P34     DM             port           output                    SRAM Data Memory Select
// P35     SPI_CLK        port           output                    SPI 
// P36     MOSI           port           output                    SPI 
// P37     TXD            port           output                    RS232/USB

//- Clock   -------------------------------------------------------------------

#define    EMR_CLOCK      7372800        //- 7,3728 MHz
#define    CPU_CLOCK      ( EMR_CLOCK / 2 )
#define    TIMER_CLOCK    ( EMR_CLOCK / 4 )

//- UART   --------------------------------------------------------------------

#define UART_T0_PRESC_19200  1
#define UART_T0_PRESC_9600   2
#define UART_T0_PRESC_4800   3
//- TIMER  --------------------------------------------------------------------

#define TMR_BIT_LOAD_T0       0x01
#define TMR_BIT_EN_T0_COUNT   0x02
#define TMR_BIT_LOAD_T1       0x04
#define TMR_BIT_EN_T1_COUNT   0x08
#define TMR_BIT_TIN_EXT_IN    0x00
#define TMR_BIT_TIN_GATE_IN   0x10
#define TMR_BIT_TIN_TRIG_NT   0x20
#define TMR_BIT_TIN_TRIG_TR   0x30
#define TMR_BIT_TOUT_T0       0x40
#define TMR_BIT_TOUT_T1       0x80
#define TMR_BIT_TOUT_SCLK     0xC0

#define PRE0_T0_SINGLE_COUNT  0x00
#define PRE0_T0_COUNT_N       0x01


#define PRE0_T1_COUNT_INTERN  0x02

#define TIMER1_START()        ( TMR |= (TMR_BIT_LOAD_T1 | TMR_BIT_EN_T1_COUNT) )
#define TIMER1_STOP()         ( TMR &= ~(TMR_BIT_LOAD_T1 | TMR_BIT_EN_T1_COUNT) )

//- Config CPLD  --------------------------------------------------------------
#define CPLD_CONF_RESET       0x00   //- Reset 0
#define CPLD_WR_EPROM_EN      0x01   //- Enable Write EPROM-Fassung fr FRAM/SRAM

//- Ports IO ------------------------------------------------------------------

//#define ADR_CRT_VGA   (*(unsigned char far volatile *)0xFF00) 

//- 0x00 = Write EPROM disable
//- 0x01 = Write EPROM enable  
#define CONF_CPLD              (*(unsigned char far volatile *)0xFE03)
#define CONF_CPLD_WR_DISABLE   0x00
#define CONF_CPLD_WR_ENABLE    0x01

//- ADC Read Data
#define ADC_IN                 (*(unsigned char far volatile *)0xFF05)

//- ADC Config Write
//- 0x00 ... 0x07 Kanal
//- 0x80 = ADC_OUT
//- 0x40 = ADC_ALE
//- 0x20 = ADC_START
#define ADC_CONFIG             (*(unsigned char far volatile *)0xFF04)
#define ADC_CONFIG_CHANNEL_1   0x00
#define ADC_CONFIG_CHANNEL_2   0x01
#define ADC_CONFIG_CHANNEL_3   0x02
#define ADC_CONFIG_CHANNEL_4   0x03
#define ADC_CONFIG_CHANNEL_5   0x04
#define ADC_CONFIG_CHANNEL_6   0x05
#define ADC_CONFIG_CHANNEL_7   0x06
#define ADC_CONFIG_CHANNEL_8   0x07

#define ADC_CONFIG_ADC_OUT     0x80
#define ADC_CONFIG_ADC_ALE     0x40
#define ADC_CONFIG_ADC_START   0x20


#define ADC_CHANNEL_CURRENT   ADC_CONFIG_CHANNEL_1
#define ADC_CHANNEL_SUPPLY    ADC_CONFIG_CHANNEL_2

#define PORT_IN_1     (*(unsigned char far volatile *)0xFF06)
#define PORT_IN_2     (*(unsigned char far volatile *)0xFF07)
#define PORT_OUT_1    (*(unsigned char far volatile *)0xFE00) //- LED
#define LED_PORT      PORT_OUT_1
#define PORT_OUT_2    (*(unsigned char far volatile *)0xFE01)
#define DISPLAY_COM   (*(unsigned char far volatile *)0xFE02)
//#define DISPLAY_RW    (*(unsigned char far volatile *)0xFE02)
#define DISPLAY_DAT   (*(unsigned char far volatile *)0xFE12)
//#define DISPLAY_DAT   (*(unsigned char far volatile *)0xFE02)

//- Speicherbereich -----------------------------------------------------------
//- nDM aktiv 
//- SRAM 0x0800 bis 0xFDFF 62976 Bytes ( UB8820 )
//- SRAM 0x1000 bis 0xFDFF 60928 Bytes ( UB8840 )
//- ADC, IN/OUT, Conf_Cpld und TFT von 0xFE00 bis 0xFFFF
//- CPLD M4A5-32/32-10JNC schaltet automatisch die Bereiche um
#if defined(ROM_RAM_SEPARATE) 

//
//#define PORT_IN_1     (*(unsigned char far volatile *)0x4400)
//#define PORT_IN_2     (*(unsigned char far volatile *)0x4300)
//#define PORT_OUT_1    (*(unsigned char far volatile *)0x4200) //- LED
//#define LED_PORT      PORT_OUT_1
//#define PORT_OUT_2    (*(unsigned char far volatile *)0x4100)
//#define DISPLAY_COM   (*(unsigned char far volatile *)0x4000)
//#define DISPLAY_DAT   (*(unsigned char far volatile *)0x4001)


#define SRAM_ANFANG   0x1000  //- SRAM begin
#define SRAM_ANFANG_P (*(unsigned char far volatile *)SRAM_ANFANG)

#else 
//- nDM nicht aktiv  ( UB8840 )
//- SRAM 0x9000 bis 0xFDFF  26879 Bytes SRAM
//- SRAM ab 0xF7FF Stack Large
//- ROM  extern 0x1000 bis 0x8FFF  24575 Bytes
//- ADC, IN/OUT und TFT von 0xF800 bis 0xFEFF
//- CPLD M4A5-32/32-10JNC schaltet automatisch die Bereiche um

#define SRAM_ANFANG   0x9000  //- SRAM begin

#endif

//- SRAM Ende gilt fr alle Speicher Config
#define SRAM_VECTOR   0xFD00  //- Vectortabelle IRQ
#define SRAM_ENDE     0xFCFF  //- SRAM ende


#define ADR_EXT_IRQ0         SRAM_VECTOR
#define ADR_EXT_IRQ1         (SRAM_VECTOR+4)
#define ADR_EXT_IRQ2         (SRAM_VECTOR+(4*2))
#define ADR_EXT_IRQ3         (SRAM_VECTOR+(4*3))
#define ADR_EXT_IRQ4         (SRAM_VECTOR+(4*4))
#define ADR_EXT_IRQ5         (SRAM_VECTOR+(4*5))

#define IRQ0_ADR_JMP         (*(unsigned char far volatile *)ADR_EXT_IRQ0)
#define IRQ1_ADR_JMP         (*(unsigned char far volatile *)ADR_EXT_IRQ1)
#define IRQ2_ADR_JMP         (*(unsigned char far volatile *)ADR_EXT_IRQ2)
#define IRQ3_ADR_JMP         (*(unsigned char far volatile *)ADR_EXT_IRQ3)
#define IRQ4_ADR_JMP         (*(unsigned char far volatile *)ADR_EXT_IRQ4)
#define IRQ5_ADR_JMP         (*(unsigned char far volatile *)ADR_EXT_IRQ5)

#define PORT_P3M_nDM         0x08
#define PORT_P3M_TXD_RXD     0x40
#define PORT_P3M_PULL_UP_P2  0x01

#define PORT_P3_P33_EOC      0x08
#define PORT_P3_P34_nDM      0x10

#define PORT_P2_TFT_RST      0x01
#define PORT_P2_DC_TFT       0x02
//#define TFT_DC_DAT()         ( P2 |=  PORT_P2_DC_TFT )
//#define TFT_DC_COM()         ( P2 &= ~PORT_P2_DC_TFT )
//- ADC  --------------------------------------------------------------------

#define ADC_VREF          5.0f
#define CURRENT_GAIN      10
#define CURRENT_RSENSE    0.24f  //- Resistor auf dem Testboard 0,24R
#define CURRENT_VSENSE    0.48f  //- max. 1,5V beim ZXCZ1021
#define CURRENT_ILAST     2.0
//- Vsense/Ilast = Rsense  0.48V/2.0A = 0.24R Sens resistor
#define CURRENT_RSENS_S   (float)(CURRENT_VSENSE/CURRENT_ILAST)
#define CURRENT_VOUT      (float)(CURRENT_GAIN*CURRENT_VSENSE)
//- Stromberechnung  (fAdc/CURRENT_GAIN)/CURRENT_RSENSE)

//- INT  --------------------------------------------------------------------

#define IMR_IRQ0_EN       (unsigned char)0x01
#define IMR_IRQ1_EN       (unsigned char)0x02
#define IMR_IRQ2_EN       (unsigned char)0x04
#define IMR_IRQ3_EN       (unsigned char)0x08
#define IMR_IRQ4_EN       (unsigned char)0x10
#define IMR_IRQ5_EN       (unsigned char)0x20

#define IMR_GLOBAL_EN     0x80

#define IMR_IRQ_TIM1      IMR_IRQ5_EN
#define IMR_IRQ_RXD       IMR_IRQ3_EN
#define IMR_IRQ_TXD       IMR_IRQ4_EN
#define IMR_IRQ_P32       IMR_IRQ0_EN
#define IMR_IRQ_P33       IMR_IRQ1_EN
#define IMR_IRQ_P31       IMR_IRQ2_EN
#define IMR_IRQ_TIM0      IMR_IRQ4_EN

#define IRQ_TIM1          IMR_IRQ5_EN
#define IRQ_RXD           IMR_IRQ3_EN
#define IRQ_TXD           IMR_IRQ4_EN
#define IRQ_P32           IMR_IRQ0_EN
#define IRQ_P33           IMR_IRQ1_EN
#define IRQ_P31           IMR_IRQ2_EN
#define IRQ_TIM0          IMR_IRQ4_EN

//- Macro ---------------------------------------------------------------------
//#define WAIT_FOR_EOC() 
//( \
//#pragma asm " \n \
//WAIT_SUPP:	TM 3, #%08     	;  testen auf 1 \n \
//				    JR Z,WAIT_SUPP	  	;  warten bis 0 \n \
//" \
//#pragma asm " \n \
//WAIT_SUPP1:	TCM 3, #%08     	;  testen auf 0 \n \
//				    JR Z,WAIT_SUPP1	  	;  warten bis 1 \n \ 
//"	\
//)
#endif
