/******************************************************************************
*                         Steffen Hofmann                                     *
*                             shotech                                         *
*******************************************************************************
*  $Source: ,v $
*-----------------------------------------------------------------------------*
*  Datum der 1. Version  :  17.09.2017 13:53:08
*  Geraete Bezeichnung   :  UB88xx TB4 
*-----------------------------------------------------------------------------*
*- $Id:  ub88xx@shotech.de
*-----------------------------------------------------------------------------*
*
*	sourcefile
*	----------
*	main
* UB8820 oder UB8840
*
******************************************************************************/


#include "hw.h"
#include "main.h"
#include "uart.h"
#include "timer.h"
#include "utils.h"
#include "adc.h"
#include "firmware.h"
#include <Limits.h>
#include <stdio.h>


const char coStartText[]={" UB8820 MON V1.0 (c)shotech\n"};
const char SRAM_TST_Ok[]={"SRAM Test Ok ! ADR: "};
const char SRAM_ADR[]={"Error Adr: "};

interrupt void Dummy(void);

int main ()
{
  unsigned short wCount;
  char cSendChar;
  BOOL bRam = TRUE;
  unsigned char far volatile *pbRam;
  char bAdcWert[8];
  
  //- Vectoren zeigen auf den externen SRAM
  //- werden fr externen ROM umgebogen
//	SET_VECTOR(IRQ0, (void (*)(void)) ADR_EXT_IRQ0);  // IRQ0 ( P32 )
//	SET_VECTOR(IRQ1, (void (*)(void)) ADR_EXT_IRQ1);  // IRQ1 ( P33 )
//	SET_VECTOR(IRQ2, (void (*)(void)) ADR_EXT_IRQ2);  // IRQ2 ( P31 )
//	SET_VECTOR(IRQ3, (void (*)(void)) ADR_EXT_IRQ3);  // IRQ3 ( P30 / RXD )
//	SET_VECTOR(IRQ4, (void (*)(void)) ADR_EXT_IRQ4);  // IRQ4 ( T0 / TXD )
//	SET_VECTOR(IRQ5, (void (*)(void)) ADR_EXT_IRQ5);  // IRQ5 ( T1 )

	SET_VECTOR(IRQ0, Dummy);  // IRQ0 ( P32 )
	SET_VECTOR(IRQ1, Dummy);  // IRQ1 ( P33 )
	SET_VECTOR(IRQ2, Dummy);  // IRQ2 ( P31 )
	SET_VECTOR(IRQ3, sio_rxd);  // IRQ3 ( P30 / RXD )
	SET_VECTOR(IRQ4, Dummy);  // IRQ4 ( T0 / TXD )
	SET_VECTOR(IRQ5, Timer1);  // IRQ5 ( T1 )
	//- Jump Adressen in den externen RAM kopieren
	//- Assembler Funktion in "copy_irq_tab.asm"
	//copyIrq( Dummy, Dummy, Dummy, sio_rxd, Dummy, Timer1, SRAM_VECTOR);
	

#if defined(ROM_RAM_SEPARATE) //- mit nDM Signal fr SRAM / ROM separate
	P3M = ( PORT_P3M_TXD_RXD | PORT_P3M_PULL_UP_P2 | PORT_P3M_nDM );	/* USART 8Bit ohne Paritt	*/
#else
	P3M = ( PORT_P3M_TXD_RXD | PORT_P3M_PULL_UP_P2 );	/* USART 8Bit ohne Paritt	*/
  P3  = PORT_P3_P34_nDM;  //- nDM auf High wenn nicht benutzt
#endif  
	/* Initialize */
	P2M = 0;			          //- Init Port 2 mode as outputs
	P2  |= PORT_P2_TFT_RST; //- TFT_RST = High
	IPR = 0x1c;			        //- Interrupt priority: IRQ0 > IRQ5
	IMR = 0x00;			        //- Disable IRQ
	
	IRQ = 0;			          //- Reset IRQ
	
	CONF_CPLD = CONF_CPLD_WR_DISABLE;   //- EPROM Write Disable ( EPROM Simulator SRAM oder FRAM )
	
	LED_PORT = 0xFF;
	//- Init ADC alles Low
	ADC_CONFIG = 0;
	
	init_uart();           //- Init UART 19200 Baud

	//init_timer1();         //- Init Timer 1 als Timetick 10ms
	
	EI();				           //- Enable Interrupts.
	//- Systemstart Meldung ber RS232 ( USB )
	sendConstChar( &coStartText[0], sizeof(coStartText));
  
  wCount = SRAM_ANFANG;
	cSendChar = RamTest( &wCount );
	if( cSendChar != 0 )
		{
			LED_PORT = 0xFC; //- Fehler
    			sendConstChar( &SRAM_ADR[0], sizeof(SRAM_ADR));
    			BinToAsciiHexW(wCount, &bAdcWert[0]);
    			bAdcWert[4]= '\n';
    			sendChar( bAdcWert, 5);
		}
		else
			{
				LED_PORT = 0xFE; //- OK!

    			sendConstChar( &SRAM_TST_Ok[0], sizeof(SRAM_TST_Ok));
     			BinToAsciiHexW(wCount, &bAdcWert[0]);
     			bAdcWert[4]= '\n';
    			sendChar( bAdcWert, 5);
    							
			}

   DI();
   //  IRQ &= 0xe7;
   //  IMR &= 0xe7;
  IMR |= IMR_IRQ_RXD; //- RXD Int 		
  EI();
   
   while(1);
//	while(1){
//
//		LED_PORT = 0x55;
//    //- Aktuelle Betriebsspannung lesen
//    bAdc = AdcReadData( ADC_CHANNEL_SUPPLY );
//      
//      BinToAsciiHexB(bAdc, bAdcWert);
//      bAdcWert[2]= '\n';
//      sendChar( bAdcWert, 3);
//    
//		for( wCount = 0; wCount < 30000; wCount++);
//
//		LED_PORT = 0xAA;
//
//    //- Aktuelle Strom lesen
//    bAdc = AdcReadData( ADC_CHANNEL_CURRENT );
//    
//      
//      BinToAsciiHexB(bAdc, bAdcWert);
//      bAdcWert[2]= '\n';
//      sendChar( bAdcWert, 3);
//      
//		for( wCount = 0; wCount < 30000; wCount++);
//	}
  
}


interrupt void Dummy(void)
{
	LED_PORT = 0xFC;
	
	
}