/******************************************************************************
*                         Steffen Hofmann                                     *
*                             shotech                                         *
*******************************************************************************
*  $Source: ,v $
*-----------------------------------------------------------------------------*
*  Datum der 1. Version  :  17.09.2017 13:53:08
*  Geraete Bezeichnung   :  UB88xx TB4 
*-----------------------------------------------------------------------------*
*- $Id:  ub88xx@shotech.de
*-----------------------------------------------------------------------------*
*
*	Headerfile
*	----------
*	UART
* UB8820 oder UB8840
*
******************************************************************************/


#include "uart.h"
#include "main.h"
//#include <ctype.h>


static TeRs232_Rx teRs232Rx;
static TeBootState teBootState;
static u16 wRxCount;
static u8 bPackCount;
static u16 wRxStart; // Start UB8820

static u16 wRxJump = 0;
static u16 wPackCount = 0;
static u16 wInBufferCount = 0;
//static u16 wRxStart = 0x1000; // Start UB8840
u8 far volatile *pStart;
void (* Startfun) (void);
//-----------------------------------------------------------------------------
//- 
//-----------------------------------------------------------------------------
void init_uart(void)
{
  //- Baud = fXtal/( 128 * PRE0 * T0 )
  //- 19200 = 7372800 / ( 128 * 1 * 3 )

//  IRQ &= 0xe7;
//  IMR &= 0xe7;
  //IMR |= IMR_IRQ_RXD; //- RXD Int
  /* 19600 baud @7,3728 Mhz */
  PRE0 = ((UART_T0_PRESC_19200<<2 ) | PRE0_T0_COUNT_N  );
  T0 = 3; //-
  //TMR = ( TMR_BIT_TOUT_T0 | TMR_BIT_LOAD_T0 | TMR_BIT_EN_T0_COUNT); //- debug
  TMR = ( TMR_BIT_LOAD_T0 | TMR_BIT_EN_T0_COUNT);
  //- Dummy Byte Senden
  //SIO = '#';

  teRs232Rx=RX_IDLE;
  teBootState = BL_IDLE;
  wRxCount = 0;
  bPackCount = 0;
  //wRxStart = 0x0800; // Start UB8820
}
//-----------------------------------------------------------------------------
//- 
//-----------------------------------------------------------------------------
void putch_ub88(char c)
{
  if (c == '\n') {
     SIO = '\r';  //- Send
#pragma asm " \n \
WAIT_PUT1:	TCM IRQ, #%10     	;  letztes Zeichen schon raus? \n \
				    JR NZ,WAIT_PUT1	  	;  warten \n \
				    AND IRQ, #%EF		    ;  reset request \n \
"
    
  }
  
    SIO = c; //- send
#pragma asm " \n \
WAIT_PUT2:	TCM IRQ, #%10     	;  letztes Zeichen schon raus? \n \
				    JR NZ,WAIT_PUT2	  	;  warten \n \
				    AND IRQ, #%EF		;  reset request \n \
"

}

//ser_o2:    ld     SIO,TEMP_1      ;output character
//ser_o1:    tcm    IRQ,#%10        ;check IRQ4
//           jr     nz,ser_o1       ;wait for complete
//           and    IRQ,#%EF        ;clear IRQ4

//-----------------------------------------------------------------------------
//- 
//-----------------------------------------------------------------------------
//u8 kbhit(void)
//{
//  if (IRQ & 0x08)	/* check for receive ready */
//    return(1);
//  return(0);
//}
//-----------------------------------------------------------------------------
//- 
//-----------------------------------------------------------------------------
//char getch(void)
//{
//  char ch;
//  while (!(IRQ & 0x08));/* Wait for receive ready */
//  ch = SIO;
//  IRQ &= ~0x08;
//  if (ch == '\r' || ch == ('\r'|0x80))
//    return '\n';
//  return(ch);
//}
//-----------------------------------------------------------------------------
//- 
//-----------------------------------------------------------------------------
void sendChar( char *cBuffer, unsigned char bCharCount)
{
	unsigned char bCount;
	
	for( bCount=0; bCount < bCharCount;bCount++ )
	{
		 putch_ub88(cBuffer[bCount]);
	}
	
}

//-----------------------------------------------------------------------------
//- 
//-----------------------------------------------------------------------------
void sendConstChar( const char *cBuffer, unsigned char bCharCount)
{
	unsigned char bCount;
	
	for( bCount=0; bCount < bCharCount;bCount++ )
	{
		if( cBuffer[bCount] != 0 )
		   putch_ub88(cBuffer[bCount]);
		else
			 break;
	}
	
}
/***********************************************************
 * Interrupt routine for sio RXD.
 ***********************************************************/
interrupt void sio_rxd(void)
{
	 u8 bRx;
	 
	 bRx = SIO;
	 
	 switch(teRs232Rx)
	 {
	 	 case RX_IDLE:
	 	 	    if( bRx == BOOT_START_ADR )
	 	 	    	{
	 	 	    		teBootState = BL_START_ADR;
	 	 	    		teRs232Rx = RX_REC;
	 	 	    		wInBufferCount = 8;
	 	 	    		putch_ub88( UART_ACK );
	 	 	    	}
	 	 	    else if( bRx == BOOT_COUNT )
	 	 	    	{
	 	 	    		teBootState = BL_COUNT;
	 	 	    		teRs232Rx = RX_REC;
	 	 	    		wInBufferCount = 8;
	 	 	    		putch_ub88( UART_ACK );
	 	 	    	}
	 	 	    else if( bRx == BOOT_CRC16 )
	 	 	    	{
	 	 	    		teBootState = BL_CRC16;
	 	 	    		teRs232Rx = RX_REC;
	 	 	    		wInBufferCount = 4;
	 	 	    		putch_ub88( UART_ACK );
	 	 	    	}
	 	 	    else if( bRx == BOOT_JUMP )
	 	 	    	{
	 	 	    		teBootState = BL_JUMP_ADR;
	 	 	    		teRs232Rx = RX_REC;
	 	 	    		wInBufferCount = 8;
	 	 	    		putch_ub88( UART_ACK );
	 	 	    	}
	 	 	    else if( bRx == BOOT_BYTE )
	 	 	    	{
	 	 	    		 teRs232Rx = RX_COUNT;
	 	 	    		 CONF_CPLD = CONF_CPLD_WR_ENABLE;
	             bPackCount = 0;
	             wInBufferCount = 2; //- immer nur ein Byte
	             putch_ub88( UART_ACK );
	 	 	    	}
	 	 	    else
	 	 	    	{
	 	 	    		putch_ub88( UART_NACK );
	 	 	    	}
	 	 	    	
	 	 	    	wPackCount = 0;
	 	 	    	//wInBufferCount = 4;
	 	 	    	 
	 	 	break;
	   case RX_REC:
	   	    if(ISVALIDHEX(bRx))
	   	    	{
	              //- laenge Bytes
	              wPackCount |= CONVERTHEX(bRx);
	   	    		  wInBufferCount--;
	   	    		  
	   	    		  if( wInBufferCount < 1 )
	   	    		  	{
	   	    		  		 switch( teBootState )
	   	    		  		 {
	                      case BL_START_ADR:
	                      	   wRxStart = wPackCount;
	                      	   putch_ub88( UART_ACK );
	                      	   teRs232Rx = RX_IDLE;
	                      	   pStart = &(*(unsigned char far volatile *)wRxStart);
	                      	   
	                      	break;
	                      case BL_COUNT:
	                      	   wRxCount = wPackCount;
	                      	   teRs232Rx = RX_IDLE;
	                      	   putch_ub88( UART_ACK );
	                      	break;
	                      case BL_CRC16:
	                      	   //- noch nicht unterstzt
	                      	   putch_ub88( UART_ACK );
	                      	   teRs232Rx = RX_IDLE;
	                      	break;
	                      case BL_JUMP_ADR:
	                      	   wRxJump = wPackCount;
	                      	   putch_ub88( UART_ACK );
	   	    		  		 		       
	   	    		  		 		       if( wRxJump >= 0x0800 )
	   	    		  		 			     {
	   	    		  		 		          DI();    //- Disable Global Interrupt
	   	    		  		 		          P01M = 0x92; //- umschalten externer Stack ( 0x96 = interner Stack )
	   	    		  		 		          IRQ = 0; //- Reset IRQ
	   	    		  		 		          IMR = 0; //- Disable alle Interrupts
	   	    		  		 		          //- Jump auf externes Programm vorbereiten
	   	    		  		 		          
	   	    		  		 		          Startfun = (void (*)(void)) wRxJump;
	   	    		  		 		          //- Jump !!
	   	    		  		 		          Startfun();
	   	    		  		 		          //TestAndJump( wRxJump+2, 0xAA99 );	   	    		  		 				
	   	    		  		 				   }
	   	    		  		 				   else
	   	    		  		 				   	{
	   	    		  		 				   		 teRs232Rx = RX_IDLE;
	   	    		  		 				   	}
	                      	break;
	                      default:
	                      	
	                      	break;   	    		  		 	
	   	    		  		 }
	   	    		  		
	   	    		  	}
	   	    		  	else
	   	    		  		{   	    		  			
	   	    		  			 wPackCount = wPackCount << 4;
	   	    		  			 putch_ub88( UART_ACK );
	   	    		  		}
	   	    	}
	   	    	else
	   	    		{
	   	    			 putch_ub88( UART_NACK );
	   	    			 teRs232Rx = RX_IDLE;
	   	    		}
	 	 	break;
	 	 case RX_COUNT:
	   	    if(ISVALIDHEX(bRx))
	   	    	{
	              //- laenge Bytes
	              bPackCount |= CONVERTHEX(bRx);
	   	    		  wInBufferCount--;
	   	    		  
	   	    		  if( wInBufferCount < 1 )
	   	    		  	{
	   	    		  		 //- Byte abspeichern
	   	    		  		 *pStart = bPackCount;
	   	    		  		 pStart++;
	   	    		  		 wRxCount--;
	   	    		  		 if( wRxCount < 1 )
	   	    		  		 	{
	   	    		  		 		putch_ub88( UART_ACK );
	   	    		  		 		CONF_CPLD = CONF_CPLD_WR_DISABLE;
	   	    		  		 		teRs232Rx = RX_IDLE;
	   	    		  		 	}
	   	    		  		 	else
	   	    		  		 		{
	   	    		  		 			  wInBufferCount = 2;
	   	    		  		 			  bPackCount = 0;
	   	    		  		 			  putch_ub88( UART_ACK );
	   	    		  		 		}
	   	    		  	}
	   	    		  	else 
	   	    		  		{
	   	    			       bPackCount = bPackCount << 4;
	   	    			       putch_ub88( UART_ACK );	   	    		  			
	   	    		  		}
	   	    	}
	   	    	else
	   	    		{
                 putch_ub88( UART_NACK );
                 teRs232Rx = RX_IDLE;
                 CONF_CPLD = CONF_CPLD_WR_DISABLE;
	   	    		}
	 	 	break;
	 	 default: putch_ub88( UART_NACK );
	 	 	break;
	 }
}